/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.portlet.orchestra;

import org.exoplatform.web.application.ApplicationMessage;
import org.exoplatform.web.application.RequestContext;
import org.exoplatform.webui.application.WebuiRequestContext;
import org.exoplatform.webui.config.annotation.ComponentConfig;
import org.exoplatform.webui.config.annotation.EventConfig;
import org.exoplatform.webui.core.UIApplication;
import org.exoplatform.webui.core.lifecycle.UIFormLifecycle;
import org.exoplatform.webui.event.Event;
import org.exoplatform.webui.event.EventListener;
import org.exoplatform.webui.form.UIFormUploadInput;
import org.ow2.orchestra.console.service.orchestra.OrchestraService;
import org.ow2.orchestra.console.webui.commons.UINovaUpload;
import org.ow2.orchestra.console.webui.commons.UINovaUploadInput;
import org.ow2.orchestra.console.webui.novaview.UINovaViewManagement;
import org.ow2.orchestra.util.Misc;

/**
 * This component upload a BAR file then deploy the the associated process(es).
 *
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 * @author Elwalid Brahim <elwalid.brahim@bull.net>
 */
@ComponentConfig(lifecycle = UIFormLifecycle.class,
                 template = "system:/groovy/novabpm/commons/webui/UploadForm.gtmpl",
                 events = {@EventConfig(listeners = UIUploadProcess.SaveActionListener.class)
                 })
public class UIUploadProcess extends UINovaUpload {

  public UIUploadProcess() throws Exception {
    super();
  }

  /**
   * This method get the uploaded data and give it to orchestra service for
   * deploy.
   *
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   */
  public void saveAction() {
    final WebuiRequestContext context = RequestContext.getCurrentInstance();
    final UIApplication uiApp = this.getAncestorOfType(UIApplication.class);
    final UINovaViewManagement viewMgt = this.getAncestorOfType(UINovaViewManagement.class);
    final UINovaUploadInput input = (UINovaUploadInput) this.getUIInput(UINovaUpload.FIELD_UPLOAD);
    try {
      // deploy the process
      final OrchestraService os = uiApp.getApplicationComponent(OrchestraService.class);
      os.deployProcess(input.getUploadDataAsStream());

      // update the grid
      viewMgt.updateView("Processes");

      // get a new upload component to get an new download id
      this.updateComponent();

      viewMgt.showView("Processes");
    } catch (final Exception e) {
      final StringBuffer error = new StringBuffer();
      Throwable t = e;
      while (t != null) {
        final String msg = t.getMessage();
        if (msg != null) {
          error.append(msg).append(Misc.LINE_SEPARATOR);
        }
        t = t.getCause();
      }
      uiApp.addMessage(new ApplicationMessage("skip", new String[] { error.toString() }, ApplicationMessage.ERROR));
      context.addUIComponentToUpdateByAjax(uiApp.getUIPopupMessages());
      this.updateComponent();
    }
  }

  /**
   * This subclass is the listener of the submit form event. It allow to upload
   * files in the server and process them (verification and deploy).
   *
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   */
  public static class SaveActionListener extends EventListener<UIUploadProcess> {

    /**
     * The execute method of the listener
     *
     * @throws Exception
     */
    @Override
    public void execute(final Event<UIUploadProcess> event) throws Exception {
      final UIUploadProcess uiUploadProcess = event.getSource();
      final UIFormUploadInput input = (UIFormUploadInput) uiUploadProcess.getUIInput(UINovaUpload.FIELD_UPLOAD);

      final UIApplication uiApp = uiUploadProcess.getAncestorOfType(UIApplication.class);
      // UINovaViewManagement viewMgt =
      // uiUploadProcess.getAncestorOfType(UINovaViewManagement.class) ;

      // Check if something has been uploaded
      if (input.getUploadResource() == null) {
        uiApp.addMessage(new ApplicationMessage("UIUploadProcess.msg.fileName-error", null));
        event.getRequestContext().addUIComponentToUpdateByAjax(uiApp.getUIPopupMessages());
        return;

      }

      // Check if the file name is good
      final String fileName = input.getUploadResource().getFileName();
      if (fileName == null || fileName.equals("")) {
        uiApp.addMessage(new ApplicationMessage("UIUploadProcess.msg.fileName-error", null));
        event.getRequestContext().addUIComponentToUpdateByAjax(uiApp.getUIPopupMessages());
        return;
      }

      uiUploadProcess.saveAction();

    }
  }
}
